param ($url, $username, $password)

$ErrorActionPreference = "Stop"
$date = Get-Date
$ProblemModels = "VO0480JFDGT", "VO0960JFDGU", "VO1920JFDGV", "VO3840JFDHA", "MO0400JFFCF", "MO0800JFFCH", "MO1600JFFCK", "MO3200JFFCL", "VO000480JWDAR", "VO000960JWDAT", "VO001920JWDAU", "VO003840JWDAV", "VO007680JWCNK", "VO015300JWCNL", "VK000960JWSSQ", "VK001920JWSSR", "VK003840JWSST", "VK007680JWSSU", "VO015300JWSSV"

$global:ReturnCode=0
$global:highsecurity=""

function IlorestLogout{
    HighSecurityRun(".\ilorest.exe --nologo logout")  | Out-Null
}

function HighSecurityRun{
    param($cl)
    $torun = $cl + $global:highsecurity
    Invoke-Expression $torun
}

function SearchForProblems {
    $jsondata = ConvertFrom-Json -InputObject (HighSecurityRun(".\ilorest.exe --nologo get Location Model PowerOnHours SerialNumber FirmwareVersion -j")| Out-String)
    foreach($model in $ProblemModels){
        foreach($harddrive in $jsondata){
            if ($harddrive.Model -eq $model){
                $harddrive.FirmwareVersion = $harddrive.FirmwareVersion.Current.VersionString
                if ($harddrive.FirmwareVersion -ne "HPD8"){
                    $global:ReturnCode=1
                    if ($harddrive.PowerOnHours -gt 32000){
                        $harddrive | Add-Member -Name "Failure" -Value "IMMINENT" -MemberType NoteProperty
                    } Elseif ($null -eq $harddrive.PowerOnHours){
                        $harddrive | Add-Member -Name "Failure" -Value "IMMINENT" -MemberType NoteProperty
                    }
                    else {
                        $harddrive | Add-Member -Name "Failure" -Value "APPROACHING" -MemberType NoteProperty
                    }
                    Write-Output "=================================================================="
                    Write-Output $harddrive | Format-List
                }
            }
        }
    }
    if ($global:ReturnCode -eq 0){
        Write-Output "No affected hard drives found"
    }
}

function GetServerInfo {
    Write-Output "Server Info:"
    Write-Output "=================================================================="
    $serverdata = ConvertFrom-Json -InputObject (HighSecurityRun(".\ilorest.exe --nologo get ServerName SerialNumber --selector Bios. -j") | Out-String)
    $hostname = ConvertFrom-Json -InputObject (HighSecurityRun(".\ilorest.exe --nologo get --selector ServiceRoot. -j") | Out-String)
    Try{
        $serverdata | Add-Member -Name "iLO HostName" -Value $hostname.Oem.Hpe.Manager[0].HostName -MemberType NoteProperty -ErrorAction SilentlyContinue
    } Catch {
        $serverdata | Add-Member -Name "iLO HostName" -Value $hostname.Oem.Hp.Manager[0].HostName -MemberType NoteProperty
    }
    
    Write-Output $serverdata | Format-List 
}

function CheckErrorCode {
    if ($LastExitCode -eq 23) {
        Write-Output "Unable to find Smart Storage Drive information."
        IlorestLogout
        exit -1
    } Elseif ($LastExitCode -eq 34){
        Write-Output "If running locally CHIF driver is required. If running remotely, -url, -username, and -password arguments are required."
        exit -1
    }
    Elseif ($LastExitCode -eq 32){
        exit -1
    }
    Elseif ($LastExitCode -eq 75){
        Write-Output "Local Higher Security Mode is detected, please include -username and -password with iLO credentials."
        exit -1
    }
    Elseif ($LastExitCode -eq 5){
        Write-Output "If running locally you must run this script as an Administrator."
        exit -1
    }
    Elseif ($LastExitCode -ne 0){
        Write-Output "A general error occurred."
        IlorestLogout
        exit -1
    }
}

Write-Output "SSD Firmware Check $date"
Write-Output "=================================================================="
if ($url -and $username -and $password){
    #Remote login
    .\ilorest.exe --nologo login $url -u $username -p $password | Out-Null
} Elseif ($username -and $password) {
    #Local High Security login
    $global:highsecurity = " -u " + $username + " -p " + $password
    .\ilorest.exe --nologo login -u $username -p $password | Out-Null
} else {
    #Local login
    .\ilorest.exe --nologo login | Out-Null
}
CheckErrorCode
GetServerInfo
HighSecurityRun(".\ilorest.exe --nologo select HpeSmartStorageDiskDrive.")
CheckErrorCode
<#Write-Output "Current Drives Found:"
HighSecurityRun(".\ilorest.exe --nologo get Location Model")
CheckErrorCode#>
Write-Output "The following discovered drives need a firmware update to Version HPD8 before PowerOnHours exceed 32,768 hours:"
SearchForProblems
IlorestLogout

exit $global:ReturnCode